@@ -12,22 +12,26 @@
 # limitations under the License.
 
 import contextlib
-from distutils import spawn
+import errno
 import logging
 import os
 import re
 import select
-import signal
 import socket
 import subprocess
 import sys
-import tenacity
 import threading
 import time
+from distutils import spawn
 
 import fixtures
+
 import jinja2
+
+import psutil
+
 import six
+
 try:
     import xattr
 except ImportError:
@@ -37,17 +41,22 @@ if six.PY3:
     fsdecode = os.fsdecode
 else:
     def fsdecode(s):
-        if isinstance(s, unicode):
+        if isinstance(s, unicode):  # noqa
             return s
         return s.decode(sys.getfilesystemencoding())
 
 
 LOG = logging.getLogger(__name__)
 
+if six.PY2:
+    # This is defined only on Python 3
+    ProcessLookupError = None  # noqa
+
 
 class Driver(fixtures.Fixture):
     def __init__(self, env_prefix="PIFPAF", templatedir=".", debug=False,
                  tmp_rootdir=None):
+        """Create a new driver."""
         super(Driver, self).__init__()
         self.env_prefix = env_prefix
         self.env = {}
@@ -90,25 +99,61 @@ class Driver(fixtures.Fixture):
             raise RuntimeError("TMPDIR must support xattr for %s" %
                                self.__class__.__name__)
 
-    def _kill(self, process):
-        pgrp = os.getpgid(process.pid)
-        process.terminate()
-        try:
-
-            self._wait(process)
-        except tenacity.RetryError:
-            LOG.warning("PID %d didn't terminate cleanly after 10 seconds, "
-                        "sending SIGKILL to its process group", process.pid)
-            # Cleanup remaining processes
-            os.killpg(pgrp, signal.SIGKILL)
-        process.wait()
-
     @staticmethod
-    @tenacity.retry(wait=tenacity.wait_fixed(1),
-                    stop=tenacity.stop_after_attempt(10),
-                    retry=tenacity.retry_if_result(lambda ret: ret is None))
-    def _wait(process):
-        return process.poll()
+    def _get_procs_of_pgid(wanted_pgid):
+        procs = []
+        for p in psutil.process_iter():
+            try:
+                pgid = os.getpgid(p.pid)
+            except OSError as e:
+                # ESRCH is returned if process just died in the meantime
+                if e.errno != errno.ESRCH:
+                    raise
+                continue
+            if pgid == wanted_pgid:
+                procs.append(p)
+        return procs
+
+    def _kill(self, parent):
+        do_sigkill = False
+        log_thread = getattr(parent, "_log_thread", None)
+        # NOTE(sileht): Add processes from process tree and process group
+        # Relying on process tree only will not work in case of
+        # parent dying prematuraly and double fork
+        # Relying on process group only will not work in case of
+        # subprocess calling again setsid()
+        procs = set(self._get_procs_of_pgid(parent.pid))
+        try:
+            procs |= set(parent.children(recursive=True))
+            procs.add(parent)
+            parent.terminate()
+        except psutil.NoSuchProcess:
+            LOG.warning("`%s` is already gone, sending SIGKILL to its process "
+                        "group", parent)
+            do_sigkill = True
+        else:
+            # Waiting for all processes to stop
+            gone, alive = psutil.wait_procs(procs, timeout=10)
+            if alive:
+                do_sigkill = True
+                LOG.warning("`%s` didn't terminate cleanly after 10 seconds, "
+                            "sending SIGKILL to its process group", parent)
+
+        if do_sigkill and procs:
+            for p in procs:
+                try:
+                    p.kill()
+                except psutil.NoSuchProcess:
+                    pass
+            gone, alive = psutil.wait_procs(procs, timeout=10)
+            if alive:
+                LOG.warning("`%s` survive SIGKILL", alive)
+
+        if log_thread:
+            # Parent process have been killed
+            log_thread.join(timeout=3)
+            if log_thread.is_alive():
+                LOG.warning("logging thread for `%s` is still alive", parent)
 
     @staticmethod
     def find_executable(filename, extra_paths):
@@ -139,6 +184,7 @@ class Driver(fixtures.Fixture):
             if not data:
                 break
             self._log_output(app, pid, data)
+        fd.close()
 
     @staticmethod
     def _log_output(appname, pid, data):
@@ -180,7 +226,7 @@ class Driver(fixtures.Fixture):
             complete_env = None
 
         try:
-            c = subprocess.Popen(
+            c = psutil.Popen(
                 command,
                 close_fds=True,
                 stdin=stdin_fd,
@@ -193,6 +239,8 @@ class Driver(fixtures.Fixture):
             raise RuntimeError(
                 "Unable to run command `%s': %s" % (b" ".join(command), e))
 
+        self.addCleanup(self._kill, c)
+
         if stdin:
             LOG.debug("%s input: %s" % (app, stdin))
             c.stdin.write(stdin)
@@ -243,6 +291,9 @@ class Driver(fixtures.Fixture):
                                  args=(app, c.pid, c.stdout,))
             t.setDaemon(True)
             t.start()
+            # Store the thread ref into the Process() to be able
+            # to clean it
+            c._log_thread = t
 
         if wait_for_port:
             for i in range(0, 10):
